import java.awt.*;
import java.lang.*;
import java.awt.event.*;
import java.util.*;
import java.awt.geom.*; 
import javax.swing.*; 
import javax.swing.event.*;

/**
 * Implementa el Panel de dibujo. Incluye los mtodos para
 * atender eventos de mouse y para relizar acciones y dibujos.
 */
public class DrawingPanel extends JPanel
{

//Atributos
/**
 * Almacena las figuras creadas por el usuario
 */
private static ArrayList figuras;
/**
 * Punto del primer click.
 */
private Point2D puntoInicial;
/**
 * Punto de Comparacin. Usualmente corresponde al primer
 * punto antes de comenzar unn arrastre o drag.
 */
private Point2D puntoComp;
/**
 * Punto Actual. Es el punto en que el mouse se encuentra
 * en este momento.
 */
private Point2D puntoAct;
/**
 * Identifica el click con botn izquierdo.
 */
private boolean oneClick;
/**
 * Identifica click con botn izquierdo.
 */
private boolean twoClick;
/**
 * Identifica un doble click.
 */
private boolean otherClick;
/**
 * Almacena Circulo mientras se dibuja, antes de pasarlo
 * al arreglo de Figuras.
 */
private Ellipse2D figsC;
/**
 * Almacena Rectngulo mientras se dibuja, antes de pasarlo
 * al arreglo de Figuras.
 */
private Rectangle2D figsR;
/**
 * Almacena la lnea mientras se dibuja, antes de pasarlo
 * al arreglo de Figuras.
 */
private Line2D figsP;
/**
 * Contiene la actual transformacin de Rotacin.
 */
private static AffineTransform tformR = new AffineTransform();
/*
 * Evita dibujar una figura cuando se estn realizaando
 * acciones de manipulacin.
 */
private boolean dontDraw;
/**
 * Identifica botn oprimido del mouse.
 */
private static boolean pressed;
/**
 * Identifica botn soltado de mouse.
 */
private boolean released;
/**
 * Identifica cambio de tamao de panel de dibujo.
 */
private boolean evento;
/**
 * Angulo de Rotacin actual
 */
private double angulo;
/**
 * Factor de Escalamiento actual.
 */
private double factor;
/**
 * Distancia actual desde punto inicial a Punto Actual.
 */
private double distAct;
/**
 * Distancia actual desde punto inicial a Punto de Comparacin.
 */
private double distComp;

private final int blanco = 255;
private boolean load;
private boolean anyClick;

private double anchorX;
private double anchorY;

//Constructor
/**
 * Constructor. Crea el panel de dibujo y agrega los 
 * manejadores de eventos de mouse.
 */
 public DrawingPanel()
 {
 figuras = new ArrayList();
 puntoInicial = null;
 figsC = new Ellipse2D.Double();
 figsR = new Rectangle2D.Double();
 figsP = new Line2D.Double();
 oneClick = false;
 twoClick = false;
 otherClick = false;
 anyClick = false;
 dontDraw = false;
 evento = false;
 load = false;
 angulo = 0;
 anchorX = 0;
 anchorY = 0;

 setOpaque(true);
 /* Un bug de Java no permite poner un borde para un panel en que se
  * va a declarar el paintComponent(). Si se des-comenta la linea de setBorder
  * ese borde tambin sufrir las AffineTransforms.
  * setBorder(BorderFactory.createLineBorder(Color.black));
  */
 setBackground(new Color(blanco, blanco, blanco));
 
 addMouseListener(new MouseHandler());
 addMouseMotionListener(new MouseMotionHandler());
 addComponentListener(new ComponentHandler());
 }
 
//Metodos
/**
 * Implementa el mtodo adecuado para el pintado de la pantalla.
 * Se incluy el cdigo para implementar las manipulaciones, adems
 * de slo dibujar las figuras.
 * @param g El componente grfico <tt>g</tt>.
 */
public void paintComponent(Graphics g)
   { super.paintComponent(g);
     Graphics2D g2 = (Graphics2D)g;

    if (load)
    { g2.rotate(angulo, anchorX, anchorY);
      tformR = g2.getTransform();
      load = false; }
         
    if (!evento) 
    { if (ButtonHandler.whatToDo() == "R") 
     { if (pressed)
       { g2.transform(tformR); pressed = false; }
       anchorX = puntoInicial.getX(); anchorY = puntoInicial.getY();
       g2.rotate(angulo, anchorX, anchorY);
       /*for(int i = 0; i < figuras.size();i++){
      Shape temp_Shape =  tformR.getRotateInstance(angulo,puntoInicial.getX(), 
           puntoInicial.getY()).createTransformedShape((Shape)figuras.get(i));
         Rfiguras.add(i,temp_Shape);}*/
       dontDraw = true; tformR = g2.getTransform(); }
      if (ButtonHandler.whatToDo() == "T")
     { if (pressed)
       { pressed = false; }
         dontDraw = true; }
      if (ButtonHandler.whatToDo() == "S")
      { if (pressed)
       { pressed = false; }
         dontDraw = true; }
     } //Cierra el evento 
      
     if ((ButtonHandler.whatToDo() != "R") 
           | (released && ButtonHandler.whatToDo() == "R"))
      { g2.transform(tformR); released = false; }
    for (int i = 0; i < figuras.size(); i++)
         g2.draw((Shape)figuras.get(i));
     if (!dontDraw)
      { g2.draw((Ellipse2D.Double)figsC); g2.draw((Rectangle2D.Double)figsR);
        g2.draw((Line2D.Double)figsP); }
     }
  
/**
 * Clase para manejar cambios en el drawing Panel
 */
private class ComponentHandler extends ComponentAdapter
{
  
/**
 * Detecta una modificacion en el tamao del Panel para manejar
 * mejor el proceso de pintado
 * @param c El <tt>componente</tt>
 */     
      public void componentResized(ComponentEvent c)
      {
        evento = true;
      }
  }

/**
 * Clase para manejar eventos de mouse
 */
private class MouseHandler extends MouseAdapter
{ private final int tres = 3;
/**
 * Atiende eventos de click de mouse
 * @param event El <tt>evento</tt> de click.
 */      
      public void mouseClicked(MouseEvent event)
      {
         evento = false;
         anyClick = true;   
         //Decodifica botones de Mouse
         if (event.getButton() == 1)
         {
            puntoInicial = event.getPoint();
          if (event.getClickCount() >= 2)
            {
                     otherClick = true;
                     oneClick = true;
                     twoClick = false; }
          else
          { oneClick = true;
            twoClick = false;
            otherClick = false; }
         }
         else if (event.getButton() == tres) 
         {  oneClick = false;
            twoClick = true;
            otherClick = false; }
         else
         { oneClick = false; twoClick = false; otherClick = false; }
      }
/**
 * Atiende evento de presin de un botn mouse.
 * @param event El <tt>evento</tt> de presionar un botn.
 */
     public void mousePressed(MouseEvent event)
      {
       pressed = true;
       anyClick = true;
      }
/**
 * Atiende evento de soltar un botn de mouse.
 * @param event El <tt>evento</tt> de soltar un botn.
 */ 
     public void mouseReleased(MouseEvent event)
     {
      released = true;
      for (int k = 0; k < figuras.size(); k++)
      { String name = figuras.get(k).getClass().getName();
        if (name.equals("MyLine"))
         { ((MyLine)figuras.get(k)).isReleased(ButtonHandler.whatToDo()); } 
        if (name.equals("MyRect"))
         { ((MyRect)figuras.get(k)).isReleased(ButtonHandler.whatToDo()); } 
        if (name.equals("MyCirc"))
         { ((MyCirc)figuras.get(k)).isReleased(ButtonHandler.whatToDo()); } 
       }
     }
    }
 
/**
 * Clase para manejar movimientos de mouse.
 */ 
 private class MouseMotionHandler implements MouseMotionListener
 {
/**
 * Maneja eventos relacionados con movimiento del mouse.
 * Dentro de este mtodo se encuentra el cdigo para dibujar
 * las distintas figuras.
 * Se invoca al mtodo repaint() para dibujar una forma.
 * @param event El <tt>evento</tt> de mover el mouse.
 */
      public void mouseMoved(MouseEvent event)
      { if (ButtonHandler.whichToDraw() == "C")
        { if (oneClick)
         { double rect; double coord; double size;
          double rectWidth = event.getX();
          double rectHeight = event.getY();
          double x = puntoInicial.getX();
          double y = puntoInicial.getY();
       if (rectWidth < rectHeight) rect = rectWidth;
          else rect = rectHeight;
       if (x < y) coord = x;
          else coord = y;
        size = Math.abs(rect - coord);
        figsC = new MyCirc(x, y, size, size);
        dontDraw = false; repaint(); }
         if (twoClick)
         { figuras.add(figsC); twoClick = false; }
       }
      if (ButtonHandler.whichToDraw() == "R")
        { if (oneClick)
          { double rectWidth = event.getX();
            double rectHeight = event.getY();
            double x = puntoInicial.getX();
            double y = puntoInicial.getY();
       figsR = new MyRect(x, y, java.lang.Math.abs(rectWidth - x), 
            java.lang.Math.abs(rectHeight - y));
       dontDraw = false; repaint(); }
       if (twoClick)
         { figuras.add(figsR); twoClick = false; }
       }
       if (ButtonHandler.whichToDraw() == "P")
       { if (oneClick && !otherClick)
         { Point2D puntoFinal = event.getPoint();
           figsP = new MyLine(puntoInicial, puntoFinal);
           dontDraw = false; repaint();  }
        if (twoClick)
        { figuras.add(figsP); twoClick = false; }
        if (otherClick)
        { figuras.add(figsP); otherClick = false; }  
       }
    } 

/**
 * Maneja eventos relacionados con arrastres del mouse.
 * Dentro de este mtodo se encuentra el cdigo para manipular
 * las distintas figuras.
 *
 * Para rotacion: Esta es una forma de obtener el angulo. Se usa la 
 * distancia entre una linea y el punto actual, donde la linea va entre 
 * el punto de click inicial y el primer punto de arrastre.
 *
 * Se invoca al mtodo repaint() para introducir una transformacin.
 * @param event El <tt>evento</tt> de mover el mouse.
 */  
   public void mouseDragged(MouseEvent event)
   { if (ButtonHandler.whatToDo() == "R")
      { if (pressed)
         { puntoComp = event.getPoint(); oneClick = false; }
     Line2D.Double linea = new Line2D.Double(puntoInicial, puntoComp);
     puntoAct = event.getPoint(); distComp = puntoInicial.distance(puntoAct);
     distAct = linea.ptLineDist(puntoAct);
     if (puntoComp.getY() > puntoAct.getY())
       { angulo = -(Math.asin(distAct / distComp)) + (2 * Math.PI); } 
     else 
       { angulo = Math.asin(distAct / distComp); }
     /*angulo = -Math.atan2(puntoAct.getX(), puntoAct.getY());*/
     repaint();
   }
      if (ButtonHandler.whatToDo() == "T")
      { if (pressed)
        { puntoComp = event.getPoint(); }
          puntoAct = event.getPoint();
          double x = puntoAct.getX() - puntoComp.getX();
          double y = puntoAct.getY() - puntoComp.getY();
          for (int k = 0; k < figuras.size(); k++)
          { String name = figuras.get(k).getClass().getName();
            translateDecision(k, name, x, y); }        
          repaint();
          }

          if (ButtonHandler.whatToDo() == "S")
         { if (pressed)
           { puntoComp = event.getPoint();
             distComp = puntoInicial.distance(puntoComp);
             oneClick = false; }
          puntoAct = event.getPoint(); distAct = puntoComp.distance(puntoAct);
          factor = (distAct / distComp);
          double mx = minCoord().getX(); double my = minCoord().getY();
        for (int k = 0; k < figuras.size(); k++)
        { String name = figuras.get(k).getClass().getName();
          scaleDecision(k, name, factor, mx, my); }
        repaint();
       }
     }
   }

protected void translateDecision (int k, String name, double x, double y)
{ if (name.equals("MyLine"))
  { ((MyLine)figuras.get(k)).translate(x, y); } 
  if (name.equals("MyRect"))
  { ((MyRect)figuras.get(k)).translate(x, y); } 
  if (name.equals("MyCirc"))
  { ((MyCirc)figuras.get(k)).translate(x, y); }
}

protected void scaleDecision (int k, String name, double factor, 
   double mx, double my)
{ if (name.equals("MyLine"))
  { ((MyLine)figuras.get(k)).scale(factor, mx, my); } 
  if (name.equals("MyRect"))
  { ((MyRect)figuras.get(k)).scale(factor, mx, my); } 
  if (name.equals("MyCirc"))
  { ((MyCirc)figuras.get(k)).scale(factor, mx, my); }
}



/**
 * Retorna el arreglo de figuras
 * @return el arreglo de Figuras.
 */ 
 public ArrayList getArrayList()
 {
  return figuras;
 }
 
/**
 * Permite ingresar las figuras cuando son ledas desde
 * archivo
 * @param entrada La figura (tipo <tt>Shape</tt>) leida desde archivo
 */ 
 public static void setArrayList(Shape entrada)
 { 
  figuras.add(entrada);
 }

/**
 * Envia los parametros de la transformacion de rotacion como un
 * arreglo. Se envia el angulo y la coordenada del punto ancla.
 * @return arrayRet El arreglo con los parametros de rotacion.
 */
public double[] getT()
 {
  final int S = 3;
  double[] arrayRet = new double[S];
  arrayRet[0] = angulo;
  arrayRet[1] = anchorX;
  arrayRet[2] = anchorY;
  return arrayRet;
 }
 
/**
 * Configura la transformacion de rotacion cuando ha
 * sido leida desde archivo
 * @param a Un arreglo con angulo y coordenadas de puntos ancla.
 */
 public void setT(double[] a)
 {
  angulo = a[0]; 
  anchorX = a[1];
  anchorY = a[2];
  load = true;
 }
 
 /**
  * Permite controlar la ejecucion de ciertos
  * comandos al invocar paintComponent(). <tt>evento</tt> es puesto
  * como verdadero cuando se hacen ajustes de componente que llaman a
  * paintComponent() pero que no son dibujo.
  * @param b <tt>evento</tt> verdadero o falso.
  */
 public void setEvento(boolean b)
 {
  evento = b;
 }
 
 /**
  * Permite que se llame el repintado del Panel de Figuras
  * desde una instancia de DrawingPanel
  */
 public void paintt()
 {
  repaint();
 }
 
 /**
  * Avisa si ha habido un click que denote edicin de figuras
  * @return Si ha sido clickado el Panel de Dibujo.
  */
 public boolean isEdited()
 {
  return anyClick;
 }
 
 /**
  * Avisa que ha sido almacenado el dibujo.
  * @param b Si el dibujo ha sido editado
  */
 public void setEdited(boolean b)
 {
  anyClick = b;
 }
 
 /**
  * Inicializa el arreglo de figuras, borrando su contenido,
  * para cargar las figuras leidas de archivo.
  * Tambin se inicializa la transformacion de rotacion dejndola
  * como identidad
  */
 public void initt()
 {
  twoClick = true;
  figuras.clear();
  figsC = new Ellipse2D.Double();
  figsR = new Rectangle2D.Double();
  figsP = new Line2D.Double();
  tformR.setToIdentity();
 }
 
 /**
  * Devuelve el punto mas arriba y a la izquierda de las figuras.
  * Se busca el Bounds2D() para cada figura y se escoge la que tenga
  * las coordenadas mas pequeas.
  * @return El punto <tt>Point2D</tt> mas arriba e izquierda en el panel.
  */
 protected Point2D minCoord()
 { double minXant = 0; double minYant = 0;
  for (int k = 0; k < figuras.size(); k++)
   { Shape tempShape = (Shape)figuras.get(k);
    Rectangle2D tempRect = tempShape.getBounds2D();
    double minX = tempRect.getMinX();
    double minY = tempRect.getMinY();
    if (k == 0)
    { minXant = minX;
      minYant = minY;
      }
    if (k > 0)
    { if (minX < minXant) 
       { minXant = minX; }
      if (minY < minYant) 
       { minYant = minY; }
      }
     }
    return new Point2D.Double(minXant, minYant);
      
 }
    
 }
