import java.util.*;
import java.awt.geom.*; 
import javax.swing.*; 
import javax.swing.event.*;
import java.io.*;

/**
 * Clase para implementar las operaciones sobre las figuras
 * Polylines. Posee metodos que sobremontan read y writeObject
 * para proveer un adecuado almacenaje de los datos
 * @author Juan Pablo Carvallo B.
 */
public class MyLine extends Line2D.Double implements Serializable
{
private double x1i;
private double y1i;
private double x2i;
private double y2i;

/**
 * Construye una linea invocando el constructor de la clase base
 * y con un par de coordenadas.
 * @param x11 Coordenada x del primer punto
 * @param y11 Coordenada y del primer punto
 * @param x22 Coordenada x del segundo punto
 * @param y22 Coordenada y del segundo punto
 */
public MyLine(double x11, double y11, double x22, double y22)
{
 super(x11, y11, x22, y22);
 x1i = x11;
 y1i = y11;
 x2i = x22;
 y2i = y22;
}

/**
 * Construye una linea invocando el constructor de la clase base
 * y con un par de puntos.
 * @param p1 Primer punto.
 * @param p2 Segudno punto.
 */
public MyLine(Point2D p1, Point2D p2)
{
 super(p1, p2);
 x1i = p1.getX();
 y1i = p1.getY();
 x2i = p2.getX();
 y2i = p2.getY();
}

/**
 * Implementacion del metodo writeObject para serializar apropiadamente
 * las instancias de clase MyLine
 * @param s El stream de salida ObjectOutputStream
 */
private void writeObject(ObjectOutputStream s) throws IOException 
{ s.defaultWriteObject();
  s.writeDouble(x1);
  s.writeDouble(y1);
  s.writeDouble(x2);
  s.writeDouble(y2);
}

/**
 * Implementacion del metodo readObject para des-serializar apropiadamente
 * las instancias de clase MyLine. Termina llamando al metodo estatico de 
 * la clase DrawingPanel para incluir la figura recien leida.
 * @param s El stream de entrada ObjectInputStream
 */
private void readObject(ObjectInputStream s) throws IOException, 
   ClassNotFoundException  
{ s.defaultReadObject();
  double xx1 = s.readDouble();
  double yy1 = s.readDouble();
  double xx2 = s.readDouble();
  double yy2 = s.readDouble();
  DrawingPanel.setArrayList(new MyLine(xx1, yy1, xx2, yy2));
}

/**
 * Transformacion de traslacion
 * @param x Coordenada de traslacion X
 * @param y Coordenada de traslacion Y
 */
public void translate (double x, double y)
{ 
  x1 = x + x1i;
  x2 = (x2i - x1i) + x1;
  y1 = y + y1i;
  y2 = (y2i - y1i) + y1; 
}

/**
 * Transformacion de escalamiento
 * @param f Factor de escalamiento
 * @param mx Coordenada X respecto a la cual se escala
 * @param my Coordenada Y respecto a la cual se escala
 */
public void scale (double f, double mx, double my)
{ 
  x1 = mx + f * (x1i - mx);
  y1 = my + f * (y1i - my); 
  x2 = mx + f * (x2i - mx);
  y2 = my + f * (y2i - my); 
}

/**
 * Actualiza los parametros privados de la figura una vez
 * que se ha efectuado la operacion de escala o traslacion
 * @param c String que entrega el codigo de la operacion efectuada
 */
public void isReleased (String c)
{  
 if (c.equals("T") | c.equals("S"))
 { x1i = x1;
   y1i = y1;
   x2i = x2;
   y2i = y2; }
}

} //End Class
