import java.util.*;
import java.awt.geom.*; 
import javax.swing.*; 
import javax.swing.event.*;
import java.io.*;

/**
 * Clase para implementar las operaciones sobre las figuras
 * Rectangulos. Posee metodos que sobremontan read y writeObject
 * para proveer un adecuado almacenaje de los datos
 * @author Juan Pablo Carvallo B.
 */
public class MyRect extends Rectangle2D.Double implements Serializable
{
private double xi;
private double yi;
private double wi;
private double hi;

/**
 * Construye un rectangulo invocando el constructor de la clase base
 * @param xx Coordenada x de la esquina superior izquierda.
 * @param yy Coordenada y de la esquina superior izquierda.
 * @param widthh Ancho del rectangulo.
 * @param heightt Alto del rectangulo.
 */
public MyRect(double xx, double yy, double widthh, double heightt)
{
 super(xx, yy, widthh, heightt);
 xi = xx;
 yi = yy;
 wi = widthh;
 hi = heightt;
}

/**
 * Implementacion del metodo writeObject para serializar apropiadamente
 * las instancias de clase MyRect
 * @param s El stream de salida ObjectOutputStream
 */
private void writeObject(ObjectOutputStream s) throws IOException 
{ s.defaultWriteObject();
  s.writeDouble(x);
  s.writeDouble(y);
  s.writeDouble(width);
  s.writeDouble(height);
}

/**
 * Implementacion del metodo readObject para des-serializar apropiadamente
 * las instancias de clase MyRect. Termina llamando al metodo estatico de 
 * la clase DrawingPanel para incluir la figura recien leida.
 * @param s El stream de entrada ObjectInputStream
 */
private void readObject(ObjectInputStream s) throws IOException, 
   ClassNotFoundException  
{ s.defaultReadObject();
  double xx = s.readDouble();
  double yy = s.readDouble();
  double wwidth = s.readDouble();
  double hheight = s.readDouble();
  DrawingPanel.setArrayList(new MyRect(xx, yy, wwidth, hheight));
}

/**
 * Transformacion de traslacion
 * @param xx Coordenada de traslacion X
 * @param yy Coordenada de traslacion Y
 */
public void translate (double xx, double yy)
{ 
 x = xx + xi;
 y = yy + yi;
}

/**
 * Transformacion de escalamiento
 * @param f Factor de escalamiento
 * @param mx Coordenada X respecto a la cual se escala
 * @param my Coordenada Y respecto a la cual se escala
 */
public void scale (double f, double mx, double my)
{
 x = mx + f * (xi - mx);
 y = my + f * (yi - my);  
 width = wi * f;
 height = hi * f;
}

/**
 * Actualiza los parametros privados de la figura una vez
 * que se ha efectuado la operacion de escala o traslacion
 * @param c String que entrega el codigo de la operacion efectuada
 */
public void isReleased (String c)
{  
  if (c.equals("T") | c.equals("S"))
    { xi = x; yi = y; }
  if (c.equals("S"))
    { wi = width; hi = height; }
}


} //End Class
